window.Gleam.initSearch([{"doc":"tagg","title":"tagg","content":"# Tagg\n\n[![Package Version](https://img.shields.io/hexpm/v/tagg)](https://hex.pm/packages/tagg)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/tagg/)\n\nTagg is an HTML templating engine written in [Gleam](https://gleam.run/) that\nuses XML/HTML tag syntax to render HTML.\n\nGoals of the project include:\n  - Familiar HTML tag syntax that editors can already highlight and format\n  - Write custom, reusable components (e.g. the `&lt;component /&gt;` tag)\n  - Include scripts (e.g. the `&lt;include /&gt;` tag)\n  - Use Gleam code alonside templates, to customize rendering\n\n**Note**: This library is still being written!\n\n## Tags\n\nHere is a usage summary of the supported tags:\n\n### Component\n\nCreate a file containing a reusable component. The context set on the parent\nfile will be passed to this component.\n\n```html\n&lt;component path=&quot;/path/to/component.html&quot; /&gt;\n```\n\n### Include\n\nIncludes content (verbatim) from another file (the contents will not be parsed).\n\n```html\n&lt;script&gt;\n  &lt;include path=&quot;/path/to/script.js&quot; /&gt;\n&lt;/script&gt;\n```\n\n### For\n\nRender a list of items.\n\n```html\n&lt;for items=&quot;people&quot; item=&quot;person&quot; index=&quot;i&quot; /&gt;\n  &lt;li&gt;@person.name&lt;/li&gt;\n&lt;/for&gt;\n```\n\n### If\n\nRender an item if a condition is true.\n\n```html\n&lt;if name=&quot;is_admin&quot; /&gt;\n  &lt;h2&gt;Users&lt;/h2&gt;\n&lt;/if&gt;\n```\n\n### Ifn\n\nRender an item if a condition is false.\n\n```html\n&lt;ifn name=&quot;is_dark_mode&quot; /&gt;\n  &lt;div&gt;This is bright.&lt;/div&gt;\n&lt;/ifn&gt;\n```\n\n## Example\n\nSee the [example Wisp app](https://github.com/jasonprogrammer/tagg_wisp_example) that uses Tagg.\n\n### router.gleam\n\n```gleam\nfn events_page(req: Request, web_context: Context) -&gt; Response {\n  // The home page can only be accessed via GET requests, so this middleware is\n  // used to return a 405: Method Not Allowed response for all other methods.\n  use &lt;- wisp.require_method(req, Get)\n\n  let context =\n    cx.dict()\n    |&gt; cx.add_list(&quot;events&quot;, [\n      cx.dict()\n        |&gt; cx.add_string(&quot;name&quot;, &quot;Muse Concert&quot;)\n        |&gt; cx.add_string(&quot;location&quot;, &quot;Los Angeles, CA&quot;),\n      cx.dict()\n        |&gt; cx.add_string(&quot;name&quot;, &quot;The Killers&quot;)\n        |&gt; cx.add_string(&quot;location&quot;, &quot;Las Vegas, NV&quot;),\n    ])\n\n  case tagg.render(web_context.tagg, &quot;events.html&quot;, context) {\n    Ok(html) -&gt; {\n      wisp.ok()\n      |&gt; wisp.html_body(string_builder.from_string(html))\n    }\n    Error(err) -&gt; {\n      io.debug(err)\n      wisp.internal_server_error()\n    }\n  }\n}\n```\n\n### events.html\n\n```html\n&lt;!doctype html&gt;\n&lt;html lang=&quot;en-US&quot;&gt;\n  &lt;head&gt;\n    &lt;meta charset=&quot;utf-8&quot; /&gt;\n    &lt;meta name=&quot;viewport&quot; content=&quot;width=device-width&quot; /&gt;\n    &lt;title&gt;Events&lt;/title&gt;\n  &lt;/head&gt;\n  &lt;body&gt;\n    &lt;component path=&quot;/events-component.html&quot;/&gt;\n  &lt;/body&gt;\n&lt;/html&gt;\n```\n\n### events-component.html\n\n```html\n&lt;h2&gt;Events&lt;/h2&gt;\n\n&lt;table border=&quot;1&quot;&gt;\n  &lt;for items=&quot;events&quot; item=&quot;event&quot; index=&quot;i&quot;&gt;\n    &lt;tr&gt;\n      &lt;td&gt;@event.name&lt;/td&gt;\n      &lt;td&gt;@event.location&lt;/td&gt;\n    &lt;/tr&gt;\n  &lt;/for&gt;\n&lt;/table&gt;\n```\n\n### Output\n\nThe output HTML (whitespace is preserved) is:\n\n```html\n&lt;!doctype html&gt;\n&lt;html lang=&quot;en-US&quot;&gt;\n  &lt;head&gt;\n    &lt;meta charset=&quot;utf-8&quot; /&gt;\n    &lt;meta name=&quot;viewport&quot; content=&quot;width=device-width&quot; /&gt;\n    &lt;title&gt;Events&lt;/title&gt;\n  &lt;/head&gt;\n  &lt;body&gt;\n    &lt;h2&gt;Events&lt;/h2&gt;\n\n&lt;table border=&quot;1&quot;&gt;\n  \n    &lt;tr&gt;\n      &lt;td&gt;Muse Concert&lt;/td&gt;\n      &lt;td&gt;Los Angeles, CA&lt;/td&gt;\n    &lt;/tr&gt;\n  \n    &lt;tr&gt;\n      &lt;td&gt;The Killers&lt;/td&gt;\n      &lt;td&gt;Las Vegas, NV&lt;/td&gt;\n    &lt;/tr&gt;\n  \n&lt;/table&gt;\n\n  &lt;/body&gt;\n&lt;/html&gt;\n```\n\nThe page looks like this:\n\n![screenshot of HTML output](./docs/images/tagg-example-html-output.png)\n\n\n## Installing\n\n```sh\ngleam add tagg\n```\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n","url":"index.html"},{"doc":"grapheme_util","title":"get_chars_matching_func","content":"pub fn get_chars_matching_func(\n  s: String,\n  acc: String,\n  match_func: fn(String) -&gt; Bool,\n) -&gt; #(String, String)\n\nSynonyms:\ngrapheme_util.get_chars_matching_func\ngrapheme_util get_chars_matching_func","url":"grapheme_util.html#get_chars_matching_func"},{"doc":"grapheme_util","title":"get_name_ignore_ws","content":"pub fn get_name_ignore_ws(s: String) -&gt; #(String, String, Int)\n\nSynonyms:\ngrapheme_util.get_name_ignore_ws\ngrapheme_util get_name_ignore_ws","url":"grapheme_util.html#get_name_ignore_ws"},{"doc":"grapheme_util","title":"get_whitespace","content":"pub fn get_whitespace(s: String) -&gt; #(String, String)\n\nSynonyms:\ngrapheme_util.get_whitespace\ngrapheme_util get_whitespace","url":"grapheme_util.html#get_whitespace"},{"doc":"grapheme_util","title":"is_alphanumeric","content":"pub fn is_alphanumeric(grapheme: String) -&gt; Bool\n\nSynonyms:\ngrapheme_util.is_alphanumeric\ngrapheme_util is_alphanumeric","url":"grapheme_util.html#is_alphanumeric"},{"doc":"grapheme_util","title":"is_alphanumeric_or_dash_or_underscore","content":"pub fn is_alphanumeric_or_dash_or_underscore(\n  grapheme: String,\n) -&gt; Bool\n\nSynonyms:\ngrapheme_util.is_alphanumeric_or_dash_or_underscore\ngrapheme_util is_alphanumeric_or_dash_or_underscore","url":"grapheme_util.html#is_alphanumeric_or_dash_or_underscore"},{"doc":"grapheme_util","title":"is_alphanumeric_or_dash_or_underscore_or_period","content":"pub fn is_alphanumeric_or_dash_or_underscore_or_period(\n  grapheme: String,\n) -&gt; Bool\n\nSynonyms:\ngrapheme_util.is_alphanumeric_or_dash_or_underscore_or_period\ngrapheme_util is_alphanumeric_or_dash_or_underscore_or_period","url":"grapheme_util.html#is_alphanumeric_or_dash_or_underscore_or_period"},{"doc":"grapheme_util","title":"is_digit","content":"pub fn is_digit(grapheme: String) -&gt; Bool\n\nSynonyms:\ngrapheme_util.is_digit\ngrapheme_util is_digit","url":"grapheme_util.html#is_digit"},{"doc":"grapheme_util","title":"is_lower_ascii","content":"pub fn is_lower_ascii(grapheme: String) -&gt; Bool\n\nSynonyms:\ngrapheme_util.is_lower_ascii\ngrapheme_util is_lower_ascii","url":"grapheme_util.html#is_lower_ascii"},{"doc":"grapheme_util","title":"is_not_double_quote","content":"pub fn is_not_double_quote(grapheme: String) -&gt; Bool\n\nSynonyms:\ngrapheme_util.is_not_double_quote\ngrapheme_util is_not_double_quote","url":"grapheme_util.html#is_not_double_quote"},{"doc":"grapheme_util","title":"is_not_gt","content":"pub fn is_not_gt(grapheme: String) -&gt; Bool\n\nSynonyms:\ngrapheme_util.is_not_gt\ngrapheme_util is_not_gt","url":"grapheme_util.html#is_not_gt"},{"doc":"grapheme_util","title":"is_not_single_quote","content":"pub fn is_not_single_quote(grapheme: String) -&gt; Bool\n\nSynonyms:\ngrapheme_util.is_not_single_quote\ngrapheme_util is_not_single_quote","url":"grapheme_util.html#is_not_single_quote"},{"doc":"grapheme_util","title":"is_not_whitespace","content":"pub fn is_not_whitespace(grapheme: String) -&gt; Bool\n\nSynonyms:\ngrapheme_util.is_not_whitespace\ngrapheme_util is_not_whitespace","url":"grapheme_util.html#is_not_whitespace"},{"doc":"grapheme_util","title":"is_upper_ascii","content":"pub fn is_upper_ascii(grapheme: String) -&gt; Bool\n\nSynonyms:\ngrapheme_util.is_upper_ascii\ngrapheme_util is_upper_ascii","url":"grapheme_util.html#is_upper_ascii"},{"doc":"grapheme_util","title":"is_whitespace","content":"pub fn is_whitespace(grapheme: String) -&gt; Bool\n\nSynonyms:\ngrapheme_util.is_whitespace\ngrapheme_util is_whitespace","url":"grapheme_util.html#is_whitespace"},{"doc":"grapheme_util","title":"grapheme_util","content":"","url":"grapheme_util.html"},{"doc":"tagg_error","title":"TaggError","content":"pub type TaggError {\n  TagAttributeParsingError(error: String)\n  ContextValueNotFoundError(error: String)\n  TemplateFileNotFoundError(error: String)\n  TagConfigMissingError\n  TagNotClosedError(error: String)\n}\n Errors that can occur while using the templating engine.\nTagAttributeParsingError(error: String)\n\n\nContextValueNotFoundError(error: String)\n\n\nTemplateFileNotFoundError(error: String)\n\n\nTagConfigMissingError\n\n\nTagNotClosedError(error: String)\n\n\nSynonyms:\ntagg_error.TaggError\ntagg_error TaggError","url":"tagg_error.html#TaggError"},{"doc":"tagg_error","title":"tagg_error","content":"","url":"tagg_error.html"},{"doc":"pre_section","title":"RawSection","content":"pub type RawSection {\n  RawTextSection(text: String, start: Int, end: Int)\n  RawTagSection(tag: RawTag)\n}\n Once we&#39;ve identified where the custom opening/closing tags exist in the\n document, we&#39;ll separate the document into &quot;sections&quot;; a section will either\n be a custom tag that we&#39;ll transform into HTML, or a fragment of text that\n does not contain any custom tags. It&#39;s called a &quot;raw&quot; section because we&#39;ve\n only done minimal parsing so far, and are going to parse each &quot;RawSection&quot;\n to create a &quot;Section&quot; with more specific attributes.\nRawTextSection(text: String, start: Int, end: Int)\n\n\nRawTagSection(tag: RawTag)\n\n\nSynonyms:\npre_section.RawSection\npre_section RawSection","url":"pre_section.html#RawSection"},{"doc":"pre_section","title":"RawTag","content":"pub type RawTag {\n  RawTag(\n    name: String,\n    start: Int,\n    end: Int,\n    is_closing_tag: Bool,\n    is_self_closing_tag: Bool,\n    attrs_text: String,\n  )\n}\n This represents the start/end of a custom tag. For example, this may be\n used to identify where a &lt;for&gt; opening tag or &lt;/for&gt; closing tag exists in\n the HTML. At this stage, we&#39;re not intending to identify opening/closing tag\n pairs or to parse the attributes inside the tags; we&#39;re just trying to\n identify where the custom opening or closing tags are in the document.\nRawTag(\n  name: String,\n  start: Int,\n  end: Int,\n  is_closing_tag: Bool,\n  is_self_closing_tag: Bool,\n  attrs_text: String,\n)\n\n\nSynonyms:\npre_section.RawTag\npre_section RawTag","url":"pre_section.html#RawTag"},{"doc":"pre_section","title":"get_raw_sections","content":"pub fn get_raw_sections(\n  s: String,\n  custom_tag_names: Set(String),\n) -&gt; Result(List(RawSection), TaggError)\n Divide the HTML document into Text or Tag sections. Identifying each\n &quot;section&quot; comes after we&#39;ve identified where the custom tags exist in the\n document.\n\n A Text section is HTML that doesn&#39;t contain any custom tags (but may contain\n variables, e.g. @person.first_name).\n\n A Tag section is HTML inside a custom tag (e.g. inside a &lt;for&gt; tag).\n\n We do not parse every HTML tag inside the document for performance and\n memory usage reasons.\nSynonyms:\npre_section.get_raw_sections\npre_section get_raw_sections","url":"pre_section.html#get_raw_sections"},{"doc":"pre_section","title":"pre_section","content":"","url":"pre_section.html"},{"doc":"section","title":"Section","content":"pub type Section {\n  Text(value: String, start: Int, context: Context)\n  Tag(\n    name: String,\n    attrs: Dict(String, String),\n    children: List(Section),\n    start: Int,\n    context: Context,\n  )\n}\n A &quot;section&quot; found in the document, that is either:\n   - A custom tag (e.g. &lt;component path=&quot;/mycomponent.html&quot;/&gt; to process\n   - A section of text that doesn&#39;t contain any custom tags. It might have\n     variables (e.g. @person.first_name) for us to replace though.\nText(value: String, start: Int, context: Context)\n\n\nTag(\n  name: String,\n  attrs: Dict(String, String),\n  children: List(Section),\n  start: Int,\n  context: Context,\n)\n\n\nSynonyms:\nsection.Section\nsection Section","url":"section.html#Section"},{"doc":"section","title":"get_attrs","content":"pub fn get_attrs(s: String) -&gt; Dict(String, String)\n Get the attributes as key-value pairs, from a custom tag. For example, if\n we&#39;re parsing this:\n \n &#39;items=&quot;people&quot; item=&quot;person&quot; index=&quot;i&quot;&#39;\n \n Then this function should return a dict with these key -&gt; value pairs:\n    items -&gt; people\n    item -&gt; person\n    index -&gt; i\n\n Any leading and trailing whitespace in the text will be removed.\nSynonyms:\nsection.get_attrs\nsection get_attrs","url":"section.html#get_attrs"},{"doc":"section","title":"get_sections","content":"pub fn get_sections(\n  raw_sections: List(RawSection),\n) -&gt; List(Section)\n\nSynonyms:\nsection.get_sections\nsection get_sections","url":"section.html#get_sections"},{"doc":"section","title":"get_sections_rec","content":"pub fn get_sections_rec(\n  sections: List(RawSection),\n  tag_stack: List(RawTag),\n  acc: List(Section),\n  child_dict: Dict(Int, List(Section)),\n) -&gt; List(Section)\n\nSynonyms:\nsection.get_sections_rec\nsection get_sections_rec","url":"section.html#get_sections_rec"},{"doc":"section","title":"section","content":"","url":"section.html"},{"doc":"tagg_config","title":"Sections","content":"pub type Sections =\n  Result(List(Section), TaggError)\n This stores Text/Tag sections with contextual values that are used to help\n render the final HTML.\n\nSynonyms:\ntagg_config.Sections\ntagg_config Sections","url":"tagg_config.html#Sections"},{"doc":"tagg_config","title":"TagConfig","content":"pub type TagConfig =\n  Dict(String, fn(Tagg, Section, Context) -&gt; Sections)\n Dict of &lt;component name&gt; -&gt; &lt;function that performs custom logic for the\n component&gt;. The names are used to find each custom element when scanning\n the HTML. Once the Tag or Text sections are all found, the list of sections\n will be iterated over, and each component&#39;s custom function will be used\n to produce any Tag/Text sections in its place in the HTML. For example, a\n &lt;for&gt; tag will be replaced by repeating its child sections.\n\nSynonyms:\ntagg_config.TagConfig\ntagg_config TagConfig","url":"tagg_config.html#TagConfig"},{"doc":"tagg_config","title":"Tagg","content":"pub type Tagg {\n  Tagg(base_dir_path: String, tag_config: TagConfig)\n}\n Configuration for the templating engine.\n The base_dir_path is used to resolve paths (e.g. &quot;/employees/employee.html&quot;)\n that appear in the template. The TagConfig is used to help find custom tags\n and replace them with custom sections in the HTML.\nTagg(base_dir_path: String, tag_config: TagConfig)\n\n\nSynonyms:\ntagg_config.Tagg\ntagg_config Tagg","url":"tagg_config.html#Tagg"},{"doc":"tagg_config","title":"tagg_config","content":"","url":"tagg_config.html"},{"doc":"tags/component_tag","title":"create_sections","content":"pub fn create_sections(\n  tagg: Tagg,\n  doc_section: Section,\n  context: Context,\n) -&gt; Result(List(Section), TaggError)\n\nSynonyms:\ntags/component_tag.create_sections\ntags/component_tag create_sections","url":"tags/component_tag.html#create_sections"},{"doc":"tags/component_tag","title":"tags/component_tag","content":"","url":"tags/component_tag.html"},{"doc":"tags/for_loop_component","title":"create_sections","content":"pub fn create_sections(\n  tagg: Tagg,\n  doc_section: Section,\n  context: Context,\n) -&gt; Result(List(Section), TaggError)\n\nSynonyms:\ntags/for_loop_component.create_sections\ntags/for_loop_component create_sections","url":"tags/for_loop_component.html#create_sections"},{"doc":"tags/for_loop_component","title":"tags/for_loop_component","content":"","url":"tags/for_loop_component.html"},{"doc":"tags/include_tag","title":"create_sections","content":"pub fn create_sections(\n  tagg: Tagg,\n  doc_section: Section,\n  context: Context,\n) -&gt; Result(List(Section), TaggError)\n\nSynonyms:\ntags/include_tag.create_sections\ntags/include_tag create_sections","url":"tags/include_tag.html#create_sections"},{"doc":"tags/include_tag","title":"tags/include_tag","content":"","url":"tags/include_tag.html"},{"doc":"tags/if_tag","title":"IfTag","content":"pub type IfTag {\n  IfTag(name: String)\n}\n This adds the &lt;if&gt; tag, which is used to conditionally render sections.\n\n Usage:\n\n &lt;if name=&quot;is_admin&quot;&gt;\n   &lt;h2&gt;Admin items&lt;/h2&gt;\n &lt;/if&gt;\nIfTag(name: String)\n\n\nSynonyms:\ntags/if_tag.IfTag\ntags/if_tag IfTag","url":"tags/if_tag.html#IfTag"},{"doc":"tags/if_tag","title":"create_sections","content":"pub fn create_sections(\n  tagg: Tagg,\n  doc_section: Section,\n  context: Context,\n) -&gt; Result(List(Section), TaggError)\n\nSynonyms:\ntags/if_tag.create_sections\ntags/if_tag create_sections","url":"tags/if_tag.html#create_sections"},{"doc":"tags/if_tag","title":"get_if_tag","content":"pub fn get_if_tag(\n  tag_name: String,\n  attrs: Dict(String, String),\n  context: Context,\n) -&gt; Result(IfTag, TaggError)\n get the attributes of the &lt;component&gt; tag, with error handling in case\n attributes don&#39;t exist\nSynonyms:\ntags/if_tag.get_if_tag\ntags/if_tag get_if_tag","url":"tags/if_tag.html#get_if_tag"},{"doc":"tags/if_tag","title":"tags/if_tag","content":"","url":"tags/if_tag.html"},{"doc":"tags/ifn_tag","title":"create_sections","content":"pub fn create_sections(\n  tagg: Tagg,\n  doc_section: Section,\n  context: Context,\n) -&gt; Result(List(Section), TaggError)\n This adds the &lt;ifn&gt; tag, which is used to conditionally render sections. It\n does the opposite of the &lt;if&gt; tag; the &#39;n&#39; stands for &#39;not&#39;.\n\n Usage:\n\n &lt;ifn name=&quot;is_admin&quot;&gt;\n   &lt;h2&gt;Non-admin items&lt;/h2&gt;\n &lt;/ifn&gt;\nSynonyms:\ntags/ifn_tag.create_sections\ntags/ifn_tag create_sections","url":"tags/ifn_tag.html#create_sections"},{"doc":"tags/ifn_tag","title":"tags/ifn_tag","content":"","url":"tags/ifn_tag.html"},{"doc":"tagg","title":"default_component_func","content":"pub fn default_component_func(\n  tagg: Tagg,\n  doc_section: Section,\n  context: Context,\n) -&gt; Result(List(Section), TaggError)\n This is a pass-through function that can serve as a component&#39;s\n transformation function. This can be used if you want to have a custom\n component (tag name) that does not have any custom logic.\nSynonyms:\ntagg.default_component_func\ntagg default_component_func","url":"tagg.html#default_component_func"},{"doc":"tagg","title":"get_html","content":"pub fn get_html(\n  children: List(Section),\n  context: Context,\n  tagg: Tagg,\n) -&gt; Result(String, TaggError)\n Transforms a list of Tag/Text sections into an HTML string.\nSynonyms:\ntagg.get_html\ntagg get_html","url":"tagg.html#get_html"},{"doc":"tagg","title":"get_html_rec","content":"pub fn get_html_rec(\n  sections: List(Section),\n  tagg: Tagg,\n  acc: String,\n) -&gt; Result(String, TaggError)\n Recursive function to get the output HTML, given a list of Tag/Text\n sections.\nSynonyms:\ntagg.get_html_rec\ntagg get_html_rec","url":"tagg.html#get_html_rec"},{"doc":"tagg","title":"render","content":"pub fn render(\n  tagg: Tagg,\n  filepath: String,\n  context: Context,\n) -&gt; Result(String, TaggError)\n Renders a template file to an HTML string, using the provided context. The\n context is a data structure that allows the consumer to pass in dynamic\n values (e.g. a list of employee records from a database) to populate the\n template.\nSynonyms:\ntagg.render\ntagg render","url":"tagg.html#render"},{"doc":"tagg","title":"replace_text_vars","content":"pub fn replace_text_vars(\n  s: String,\n  context: Context,\n  acc: String,\n) -&gt; String\n Find template variables (e.g. @employee.first_name) in the HTML string and\n replace them with the values in the context, if they exist. If they don&#39;t\n exist, then leave the variable intact in the output HTML.\nSynonyms:\ntagg.replace_text_vars\ntagg replace_text_vars","url":"tagg.html#replace_text_vars"},{"doc":"tagg","title":"tagg","content":"","url":"tagg.html"}]);